/*
 * fill sun framebuffer with taac-1 key colour
 *
 * Author:
 * Arne Ludwig
 */

#include <stdio.h>
#include <pixrect/pixrect_hs.h>
#include <taac1/taio.h>

#define	UNIQUE0		0x00		/* hopefully black, don't care */
#define	UNIQUE		0xAA		/* some unused index */

/*
 * IMPORTANT
 *
 * You must define your key colour here!
 * Alternatively, you can try the -i option, which gets it from
 * the configuration files. Because this uses ta_init, you cannot
 * use fillfb -i while Xtaac is running, as it interferes with its
 * operation. On my system ta_get_insert_color returns wrong colour
 * values, so this mode seems not very useful anyway.
 */
#define	TAAC_BLUE	0x73
#define	TAAC_GREEN	0x2c
#define	TAAC_RED	0x34

extern char *optarg;
static char usage[] = "usage: %s [-i] [-v]\n";
static char sccsid[] = "@(#)backgr.c	1.3  91/03/22";

main(argc,argv)
int argc;
char *argv[];
{
	extern char *malloc(), *calloc();
	long *data;
	char *p;
	Pixrect *screen,*mpr,*pr;
	char buf[1000];
	colormap_t colormap;
	FILE *fp;
	char *fn;
	int errflag,c;
	int xp,yp;
	unsigned char red, green, blue;
	unsigned char ored, ogreen, oblue;
	int tac_red, tac_green, tac_blue, tac_aperture;
	register int i;
	int use_ta_init, verbose;
	TA_HANDLE *tahndl;

	/* defaults */
	use_ta_init = 0;
	verbose = 0;

	errflag = 0;
	while ((c=getopt (argc,argv,"iv")) != EOF)
		switch (c)
		{
		case 'i':
			use_ta_init++;
			break;
		case 'v':
			verbose++;
			break;
		case '?':
			errflag++;
		}

	if (errflag)
	{
		fprintf (stderr,usage,argv[0]);
		exit (1);
	}

	if (use_ta_init)
	{
		if (!(tahndl = ta_open (0)))
		{
			fprintf (stderr,"no TAAC-1\n");
			exit (2);
		}
		ta_init (tahndl);	/* don't do this, if Xtaac running */
		ta_get_insert_color (tahndl,
			&tac_red, &tac_green, &tac_blue, &tac_aperture);
	}
	else
	{
		tac_red = TAAC_RED;
		tac_green = TAAC_GREEN;
		tac_blue = TAAC_BLUE;
		tac_aperture = 255;
	}
	if (verbose)
	{
		printf ("Key colour:\n");
		printf ("  red    = %02x\n", tac_red);
		printf ("  green  = %02x\n", tac_green);
		printf ("  blue   = %02x\n", tac_blue);
		printf ("  aper   = %02x\n", tac_aperture);
		getchar ();
	}

	if (!(screen = pr_open ("/dev/fb")))
	{
		fprintf (stderr,"no frame buffer\n");
		exit (2);
	}

	if ((data = calloc (screen->pr_size.x, screen->pr_size.y)) == NULL) {
		perror ("malloc");
		exit (2);
	}

	memset (data, UNIQUE0, screen->pr_size.x * screen->pr_size.y);

	p = (char *) data;
	for (i=0; i < screen->pr_size.y; i++)
	{
		memset (p, UNIQUE, 1024);
		p += screen->pr_size.x;
	}

	red	= tac_red;
	green	= tac_green;
	blue	= tac_blue;
	if (pr_getcolormap (screen, UNIQUE, 1, &ored, &ogreen, &oblue) == -1) {
		perror ("pr_getcolormap");
		exit (1);
	}

	pr_putcolormap (screen, UNIQUE, 1, &red, &green, &blue);

	if ((mpr = mem_point (screen->pr_size.x, screen->pr_size.y,
		screen->pr_depth, data)) == NULL) {
		perror ("mem_create");
		exit (1);
	}
	pr_rop (screen,0,0,screen->pr_size.x,screen->pr_size.y,PIX_SRC,mpr,0,0);
}
