/*-
 * taac.c --
 *	Functions to support the sun taac-1 board as a memory frame buffer.
 */

/*
 * Copyright (c) 1990, 1991, Arne Ludwig, Buchholz.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted provided
 * that: (1) source distributions retain this entire copyright notice and
 * comment, and (2) distributions including binaries display the following
 * acknowledgement: "This product includes software developed by Arne Ludwig"
 * in the documentation or other materials provided with the distribution and
 * in all advertising materials mentioning features or use of this software.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/************************************************************
Copyright 1987 by Sun Microsystems, Inc. Mountain View, CA.

                    All Rights Reserved

Permission  to  use,  copy,  modify,  and  distribute   this
software  and  its documentation for any purpose and without
fee is hereby granted, provided that the above copyright no-
tice  appear  in all copies and that both that copyright no-
tice and this permission notice appear in  supporting  docu-
mentation,  and  that the names of Sun or MIT not be used in
advertising or publicity pertaining to distribution  of  the
software  without specific prior written permission. Sun and
M.I.T. make no representations about the suitability of this
software for any purpose. It is provided "as is" without any
express or implied warranty.

SUN DISCLAIMS ALL WARRANTIES WITH REGARD TO  THIS  SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
NESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL SUN BE  LI-
ABLE  FOR  ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,  DATA  OR
PROFITS,  WHETHER  IN  AN  ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
THE USE OR PERFORMANCE OF THIS SOFTWARE.

********************************************************/

#ifndef	lint
static char sccsid[] = "@(#)taac.c	1.1 (kogs26) 91/02/03";
#endif

#include    "stdio.h"		/* temporary */
#include    "taac.h"

#include    <sys/mman.h>
#include    <struct.h>
#include    "colormap.h"
#include    "colormapst.h"
#include    "resource.h"

#ifndef _MAP_NEW
extern caddr_t valloc();
#else
extern caddr_t mmap();
#endif	_MAP_NEW

typedef struct taac {
    union byteplane *image;		/* the 8-bit memory */
} taac, Rec, *Ptr;

/*
 * Hmm.
 */
#define	TA1_HEIGHT	1024
#define	TA1_WIDTH	1024
#define	TA1_DEPTH	32

struct	taacmemfb {
	union byteplane {			/* pixel mode memory */
		long pixel[TA1_HEIGHT][TA1_WIDTH];
	} pixplane;
};

#define _IMAGE(fb)	    ((caddr_t)((fb).image))
#define _IMAGELEN	    (sizeof(union byteplane))

static int  taacScreenIndex;

extern int TellLostMap(), TellGainedMap();

static void
taacUpdateColormap(fb, index, count, rmap, gmap, bmap)
    Ptr 	  fb;
    int		  index, count;
    u_char	  *rmap, *gmap, *bmap;
{
#ifdef SUN_WINDOWS
    if (taacUseSunWindows()) {
	static Pixwin *pw = 0;

	if (! pw) {
	    if ( ! (pw = pw_open(windowFd)) )
		FatalError( "taacUpdateColormap: pw_open failed\n" );
	    pw_setcmsname(pw, "X.V11");
	}
	if one of the maps is "none" -> no change
	pw_putcolormap(
	    pw, index, count, &rmap[index], &gmap[index], &bmap[index]
	);
    }
#endif SUN_WINDOWS

    if (tintSetColourmap (index, count, rmap, gmap, bmap) == -1) {
	perror("taacUpdateColormap");
	FatalError( "taacUpdateColormap: tintSetColourmap failed\n" );
    }
}

/*-
 *-----------------------------------------------------------------------
 * taacSaveScreen --
 *	Preserve the color screen by turning on or off the video
 *
 * Results:
 *	None.
 *
 * Side Effects:
 *	Video state is switched
 *
 *-----------------------------------------------------------------------
 */
static Bool
taacSaveScreen (pScreen, on)
    ScreenPtr	  pScreen;
    Bool    	  on;
{
    int		state = 1;

    if (on != SCREEN_SAVER_ON) {
	SetTimeSinceLastInputEvent();
	state = 1;
    } else {
	state = 0;
    }
    tintVideo (state);
    return( TRUE );
}

/*-
 *-----------------------------------------------------------------------
 * taacCloseScreen --
 *	called to ensure video is enabled when server exits.
 *
 * Results:
 *	Screen is unsaved.
 *
 * Side Effects:
 *	None
 *
 *-----------------------------------------------------------------------
 */
/*ARGSUSED*/
static Bool
taacCloseScreen(i, pScreen)
    int		i;
    ScreenPtr	pScreen;
{
    Bool    ret;

    pScreen->CloseScreen = (Bool (*)()) pScreen->devPrivates[taacScreenIndex].ptr;
    ret = (*pScreen->CloseScreen) (i, pScreen);
    sunFbs[pScreen->myNum].fbPriv = NULL;
    (void)(*pScreen->SaveScreen)(pScreen, SCREEN_SAVER_OFF);
    return ret;
}

/*-
 *-----------------------------------------------------------------------
 * taacInstallColormap --
 *	Install given colormap.
 *
 * Results:
 *	None
 *
 * Side Effects:
 *	All clients requesting ColormapNotify are notified
 *
 *-----------------------------------------------------------------------
 */
static void
taacInstallColormap(cmap)
    ColormapPtr	cmap;
{
    register int i;
    register Entry *pent;
    register VisualPtr pVisual = cmap->pVisual;
    u_char	  rmap[256], gmap[256], bmap[256];
    fbFd    	  *fb = &sunFbs[cmap->pScreen->myNum];

    if (cmap == (ColormapPtr)fb->fbPriv)
	return;
    if (fb->fbPriv)
	WalkTree(cmap->pScreen, TellLostMap,
		 (pointer) &(((ColormapPtr)fb->fbPriv)->mid));
    if ((pVisual->class | DynamicClass) == DirectColor) {
	for (i = 0; i < 256; i++) {
	    rmap[i] = cmap->red[i].co.local.red >> 8;
	    gmap[i] = cmap->green[i].co.local.green >> 8;
	    bmap[i] = cmap->blue[i].co.local.blue >> 8;
	}
    } else {
	for (i = 0, pent = cmap->red;
	     i < pVisual->ColormapEntries;
	     i++, pent++) {
	    if (pent->fShared) {
		rmap[i] = pent->co.shco.red->color >> 8;
		gmap[i] = pent->co.shco.green->color >> 8;
		bmap[i] = pent->co.shco.blue->color >> 8;
	    }
	    else {
		rmap[i] = pent->co.local.red >> 8;
		gmap[i] = pent->co.local.green >> 8;
		bmap[i] = pent->co.local.blue >> 8;
	    }
	}
    }
    fb->fbPriv = (pointer)cmap;
    taacUpdateColormap((Ptr)fb->fb, 0, 256, rmap, gmap, bmap);
    WalkTree(cmap->pScreen, TellGainedMap, (pointer) &(cmap->mid));
}

/*-
 *-----------------------------------------------------------------------
 * taacUninstallColormap --
 *	Uninstall given colormap.
 *
 * Results:
 *	None
 *
 * Side Effects:
 *	All clients requesting ColormapNotify are notified
 *
 *-----------------------------------------------------------------------
 */
static void
taacUninstallColormap(cmap)
    ColormapPtr	cmap;
{
    if (cmap == (ColormapPtr)sunFbs[cmap->pScreen->myNum].fbPriv) {
	Colormap defMapID = cmap->pScreen->defColormap;

	if (cmap->mid != defMapID) {
	    ColormapPtr defMap;

	    defMap = (ColormapPtr) LookupIDByType(defMapID, RT_COLORMAP);
	    if (defMap)
		(*cmap->pScreen->InstallColormap)(defMap);
	    else
	        ErrorF("taac: Can't find default colormap\n");
	}
    }
}

/*-
 *-----------------------------------------------------------------------
 * taacListInstalledColormaps --
 *	Fills in the list with the IDs of the installed maps
 *
 * Results:
 *	Returns the number of IDs in the list
 *
 * Side Effects:
 *	None
 *
 *-----------------------------------------------------------------------
 */
/*ARGSUSED*/
static int
taacListInstalledColormaps(pScreen, pCmapList)
    ScreenPtr	pScreen;
    Colormap	*pCmapList;
{
    *pCmapList = ((ColormapPtr)sunFbs[pScreen->myNum].fbPriv)->mid;
    return (1);
}


/*-
 *-----------------------------------------------------------------------
 * taacStoreColors --
 *	Sets the pixels in pdefs into the specified map.
 *
 * Results:
 *	None
 *
 * Side Effects:
 *	None
 *
 *-----------------------------------------------------------------------
 */
static void
taacStoreColors(pmap, ndef, pdefs)
    ColormapPtr	pmap;
    int		ndef;
    xColorItem	*pdefs;
{
    u_char	rmap[256], gmap[256], bmap[256], *none;
    register VisualPtr pVisual = pmap->pVisual;
    register int i, j, flags;
    Ptr fb;

    if (pmap != (ColormapPtr)sunFbs[pmap->pScreen->myNum].fbPriv)
	return;

    fb = (Ptr)sunFbs[pmap->pScreen->myNum].fb;
    none = (u_char *) 0;
    while (ndef--) {
	i = pdefs->pixel;
	flags = pdefs->flags;

	if (flags & DoRed)
	{
	    j = (i & pVisual->redMask) >> pVisual->offsetRed;
	    rmap[j] = pdefs->red >> 8;
	    taacUpdateColormap(fb, j, 1, rmap, none, none);
	}
	if (flags & DoGreen)
	{
	    j = (i & pVisual->greenMask) >> pVisual->offsetGreen;
	    gmap[j] = pdefs->green >> 8;
	    taacUpdateColormap(fb, j, 1, none, gmap, none);
	}
	if (flags & DoBlue)
	{
	    j = (i & pVisual->blueMask) >> pVisual->offsetBlue;
	    bmap[j] = pdefs->blue >> 8;
	    taacUpdateColormap(fb, j, 1, none, none, bmap);
	}

	pdefs++;
    }
}

/*-
 *-----------------------------------------------------------------------
 * taacInit --
 *	Attempt to find and initialize all available frame buffers
 *
 * Results:
 *	TRUE if everything went ok. FALSE if not.
 *
 * Side Effects:
 *	Most of the elements of the ScreenRec are filled in. Memory is
 *	allocated for the frame buffer and the buffer is mapped. The
 *	video is enabled for the frame buffer...
 *
 *-----------------------------------------------------------------------
 */
/*ARGSUSED*/
static Bool
taacInit (index, pScreen, argc, argv)
    int	    	  index;    	/* The index of pScreen in the ScreenInfo */
    ScreenPtr	  pScreen;  	/* The Screen to initialize */
    int	    	  argc;	    	/* The number of the Server's arguments. */
    char    	  **argv;   	/* The arguments themselves. Don't change! */
{
    if (!cfb32ScreenInit (pScreen,
			(pointer)((Ptr)sunFbs[index].fb)->image,
			sunFbs[index].info.fb_width,
			sunFbs[index].info.fb_height,
			monitorResolution, monitorResolution,
			sunFbs[index].info.fb_width))
	return (FALSE);

    pScreen->SaveScreen =   	    	taacSaveScreen;
    pScreen->devPrivates[taacScreenIndex].ptr = (pointer) pScreen->CloseScreen;
    pScreen->CloseScreen = taacCloseScreen;

#ifndef STATIC_COLOR
    pScreen->InstallColormap = 	    	taacInstallColormap;
    pScreen->UninstallColormap =    	taacUninstallColormap;
    pScreen->ListInstalledColormaps = 	taacListInstalledColormaps;
    pScreen->StoreColors =  	    	taacStoreColors;
#endif /* STATIC_COLOR */

    taacSaveScreen( pScreen, SCREEN_SAVER_FORCER );
    return (sunScreenInit(pScreen) && cfb32CreateDefColormap(pScreen));
}


/*-
 *-----------------------------------------------------------------------
 * taacProbe --
 *	Attempt to find and initialize the framebuffer
 *
 * Results:
 *	TRUE if everything went ok. FALSE if not.
 *
 * Side Effects:
 *	the buffer is mapped.
 *
 *-----------------------------------------------------------------------
 */
Bool
taacProbe (pScreenInfo, index, fbNum, argc, argv)
    ScreenInfo	  *pScreenInfo;	/* The screenInfo struct */
    int	    	  index;    	/* The index of pScreen in the ScreenInfo */
    int	    	  fbNum;    	/* Index into the taacFbData array */
    int	    	  argc;	    	/* The number of the Server's arguments. */
    char    	  **argv;   	/* The arguments themselves. Don't change! */
{
    int		i;
    int         fd;
    struct fbtype	fbType;
    static Rec	  fb;

#if 0
    for (i = 0; i < argc; i++) {
	if (strcmp (argv[i], "-someoption") == 0) {
	    return someProbe (pScreenInfo, index, fbNum, argc, argv);
	}
    }
#endif

/*
 * XXX
 * This stuff seems fairly obsolete...
 */
#define	FBTYPE_SUNTAAC	FBTYPE_SUNFAST_COLOR	/* as long as it isn't BW2 */
    if ((fd = sunOpenFrameBuffer(FBTYPE_SUNTAAC, &fbType,
				 index, fbNum, argc, argv)) < 0)
    {
	printf ("cannot sunOpenFrameBuffer!\n");
	return FALSE;
    }

    fbType.fb_height = TA1_HEIGHT;
    fbType.fb_width = TA1_WIDTH;
    fbType.fb_depth = TA1_DEPTH;
    fbType.fb_cmsize = 256;
    fbType.fb_size = TA1_HEIGHT*TA1_WIDTH*TA1_DEPTH/8;
#ifdef	_MAP_NEW

#define BASE		0
/*
 * Parts of this code should be moved to taacCreate????
 */
{
	TA_HANDLE *tahndl;	/* Handle returned from ta_open */
	extern TA_HANDLE *tintStartup();	/* routine to open, init TAAC */
	int ta_set_display();
	int ioflag;		/* value for handshaking flag */
	int n, tastatus;
	register int i;
	extern float a[], b[];
	float c;
	int *pt;

	/* open, initialize, load, run TAAC */
	tahndl = tintStartup();
	if (tahndl == NULL)
	{
		Error ("tintStartup returned NULL\n");
		goto bad;
	}

	/* get input and send commands to TAAC */
	if (ta_set_display(tahndl, TA_ON))
	{
		Error("error turning on display\n");
		goto bad;
	}

	if ((fb.image = (union byteplane *) ta_map(tahndl, BASE)) == NULL) {
		  Error ("Mapping taac.image");
		  goto bad;
	}

#if 0
	if ((fb.image = malloc (8*_IMAGELEN)) == NULL) {
		perror ("malloc shadow memory");
		goto bad;
	}
#endif

	ta_use_map_2d (tahndl, BASE);

#if DEBUG
	/*
	 * visual test to see if fb.image is ok
	 */
 	{
	register int i, x, y;
	x = y = 0;
	for (i=0; i < 0x100000; i++)
		*((long *)fb.image + i) = ((y>>2) + ((x>>2)<<8));
		x++;
		if (x >= 1024) {
			x = 0; y++;
		}
	}
	printf ("fb.image = %08lx\n", fb.image);
#endif

	goto ok;
}
#else
    Error ("This server will run only on _MAP_NEW machines!!");
    Error ("");
    Error ("Contact ludwiga@informatik.uni-hamburg.de or");
    Error ("change ddx/taac/taac.c yourself.");
    goto bad;
#endif	_MAP_NEW
bad:
    (void) close (fd);
    return FALSE;

ok:
    /*
     * Enable all planes
     */
    sunFbs[index].fd = fd;
    sunFbs[index].info = fbType;
    sunFbs[index].fb = (pointer) &fb;
    sunFbs[index].EnterLeave = NoopDDA;
    taacSupportsDepth32 = TRUE;
    return TRUE;
}

/*ARGSUSED*/
Bool
taacCreate(pScreenInfo, argc, argv)
    ScreenInfo	  *pScreenInfo;
    int	    	  argc;
    char    	  **argv;
{
    if (sunGeneration != serverGeneration)
    {
	taacScreenIndex = AllocateScreenPrivateIndex();
	if (taacScreenIndex < 0)
	    return FALSE;
    }
    return (AddScreen(taacInit, argc, argv) >= 0);
}
